from sator_app.bootstrap.resolution import ResolutionBuilder
from sator_app.bootstrap.extraction import ExtractionBuilder
from sator_app.bootstrap.annotation import AnnotationBuilder
from sator_app.bootstrap.analysis import AnalysisBuilder



from unittest.mock import MagicMock

from sator_app.bootstrap.analysis import AnalysisBuilder
from sator_core.use_cases.analysis.attributes import (
    PatchAttributesAnalysis,
    ProductAttributesAnalysis,
    VulnerabilityAttributesAnalysis,
)


def test_analysis_builder_creates_patch_attributes_analysis():
    # Arrange
    mock_storage = MagicMock()
    mock_patch_analyzer = MagicMock()
    mock_product_repos = [MagicMock()]
    mock_diff_classifier = MagicMock()
    mock_oss_gateway = MagicMock()

    builder = AnalysisBuilder(
        prod_repos=mock_product_repos,
        diff_classifier=mock_diff_classifier,
        patch_attrs_analyzer=mock_patch_analyzer,
        storage_port=mock_storage,  # this and oss_gateway likely set in BaseBuilder
        oss_gateway=mock_oss_gateway
    )

    # Act
    patch_analysis = builder.create_patch_attributes_analysis()
    product_analysis = builder.create_product_attributes_analysis()
    vulnerability_analysis = builder.create_vulnerability_attributes_analysis()

    # Assert
    assert isinstance(patch_analysis, PatchAttributesAnalysis)
    assert patch_analysis.storage_port is mock_storage
    assert patch_analysis.patch_analyzer is mock_patch_analyzer
    assert isinstance(product_analysis, ProductAttributesAnalysis)
    assert product_analysis.oss_gateway is mock_oss_gateway
    assert product_analysis.storage_port is mock_storage
    assert isinstance(vulnerability_analysis, VulnerabilityAttributesAnalysis)
    assert vulnerability_analysis.product_repositories == mock_product_repos
    assert vulnerability_analysis.storage_port is mock_storage



from typing import List

from sator_app.bootstrap.base import BaseBuilder

from sator_core.ports.driven.repositories.product import ProductRepositoryPort
from sator_core.ports.driven.repositories.vulnerability import VulnerabilityRepositoryPort

from sator_core.use_cases.resolution.metadata import VulnerabilityMetadataResolution, ProductMetadataResolution
from sator_core.use_cases.resolution.references import (
    ProductReferencesResolution, VulnerabilityReferencesResolution, PatchReferencesResolution
)


class ResolutionBuilder(BaseBuilder):
    def __init__(
            self, vuln_repos: List[VulnerabilityRepositoryPort], prod_repos: List[ProductRepositoryPort], **kwargs
    ):
        super().__init__(**kwargs)
        self.vulnerability_repositories = vuln_repos
        self.product_repositories = prod_repos

    def create_vulnerability_metadata_resolution(self) -> VulnerabilityMetadataResolution:
        return VulnerabilityMetadataResolution(
            vuln_repositories=self.vulnerability_repositories,
            storage_port=self.storage_port
        )

    def create_vulnerability_references_resolution(self) -> VulnerabilityReferencesResolution:
        return VulnerabilityReferencesResolution(
            vulnerability_repositories=self.vulnerability_repositories,
            storage_port=self.storage_port
        )

    def create_product_metadata_resolution(self) -> ProductMetadataResolution:
        return ProductMetadataResolution(
            product_repositories=self.product_repositories,
            storage_port=self.storage_port
        )

    def create_product_references_resolution(self) -> ProductReferencesResolution:
        return ProductReferencesResolution(
            product_repositories=self.product_repositories,
            storage_port=self.storage_port
        )

    def create_patch_references_resolution(self) -> PatchReferencesResolution:
        return PatchReferencesResolution(
            oss_gateway=self.oss_gateway,
            storage_port=self.storage_port
        )




from sator_app.bootstrap.base import BaseBuilder

from sator_core.ports.driven.extraction.attributes.patch import PatchAttributesExtractorPort
from sator_core.ports.driven.extraction.attributes.vulnerability import VulnerabilityAttributesExtractorPort


from sator_core.use_cases.extraction.attributes import (
    ProductAttributesExtraction, VulnerabilityAttributesExtraction, PatchAttributesExtraction
)


class ExtractionBuilder(BaseBuilder):
    def __init__(
            self, patch_attrs_extractor: PatchAttributesExtractorPort,
            vuln_attrs_extractor: VulnerabilityAttributesExtractorPort, **kwargs
    ):
        super().__init__(**kwargs)

        self.patch_attributes_extractor = patch_attrs_extractor
        self.vulnerability_attributes_extractor = vuln_attrs_extractor

    def create_product_attributes_extraction(self) -> ProductAttributesExtraction:
        return ProductAttributesExtraction(
            storage_port=self.storage_port
        )

    def create_vulnerability_attributes_extraction(self) -> VulnerabilityAttributesExtraction:
        return VulnerabilityAttributesExtraction(
            attributes_extractor=self.vulnerability_attributes_extractor,
            storage_port=self.storage_port
        )

    def create_patch_attributes_extraction(self) -> PatchAttributesExtraction:
        return PatchAttributesExtraction(
            oss_gateway=self.oss_gateway,
            attributes_extractor=self.patch_attributes_extractor,
            storage_port=self.storage_port
        )



from typing import List
from sator_app.bootstrap.base import BaseBuilder

from sator_core.ports.driven.classifiers.diff import DiffClassifierPort
from sator_core.ports.driven.repositories.product import ProductRepositoryPort
from sator_core.ports.driven.analyzers.patch import PatchAttributesAnalyzerPort


from sator_core.use_cases.analysis.attributes import (
    PatchAttributesAnalysis, ProductAttributesAnalysis, VulnerabilityAttributesAnalysis
)


class AnalysisBuilder(BaseBuilder):
    def __init__(
            self, prod_repos: List[ProductRepositoryPort], diff_classifier: DiffClassifierPort,
            patch_attrs_analyzer: PatchAttributesAnalyzerPort, **kwargs
    ):
        super().__init__(**kwargs)
        self.product_repositories = prod_repos
        self.patch_attributes_analyzer = patch_attrs_analyzer
        self.diff_classifier = diff_classifier

    def create_patch_attributes_analysis(self) -> PatchAttributesAnalysis:
        return PatchAttributesAnalysis(
            patch_analyzer=self.patch_attributes_analyzer,
            storage_port=self.storage_port
        )

    def create_product_attributes_analysis(self) -> ProductAttributesAnalysis:
        return ProductAttributesAnalysis(
            oss_gateway=self.oss_gateway,
            storage_port=self.storage_port
        )

    def create_vulnerability_attributes_analysis(self) -> VulnerabilityAttributesAnalysis:
        return VulnerabilityAttributesAnalysis(
            prod_repos=self.product_repositories,
            storage_port=self.storage_port
        )



from sator_core.models.vulnerability import VulnerabilityLocator

from sator_core.use_cases.resolution.metadata.vulnerability import VulnerabilityMetadataResolution
from sator_core.use_cases.resolution.references.vulnerability import VulnerabilityReferencesResolution
from sator_core.use_cases.annotation.attributes.vulnerability import VulnerabilityAttributesAnnotation
from sator_core.use_cases.extraction.attributes.vulnerability import VulnerabilityAttributesExtraction
from sator_core.use_cases.analysis.attributes.vulnerability import VulnerabilityAttributesAnalysis


class VulnerabilityProcessingService:
    def __init__(
            self, vulnerability_annotation: VulnerabilityAttributesAnnotation,
            vulnerability_extraction: VulnerabilityAttributesExtraction,
            vulnerability_analysis: VulnerabilityAttributesAnalysis,
            vulnerability_metadata: VulnerabilityMetadataResolution,
            vulnerability_references: VulnerabilityReferencesResolution
    ):
        self.vulnerability_metadata = vulnerability_metadata
        self.vulnerability_references = vulnerability_references
        self.vulnerability_annotation = vulnerability_annotation
        self.vulnerability_extraction = vulnerability_extraction
        self.vulnerability_analysis = vulnerability_analysis

    # TODO: should return something that assembles all the information
    def process_vulnerability(self, vulnerability_id: str) -> VulnerabilityLocator | None:
        metadata = self.vulnerability_metadata.resolve_vulnerability_metadata(vulnerability_id)

        if metadata:
            print(metadata)
            description = self.vulnerability_metadata.resolve_vulnerability_description(vulnerability_id)
            print(description)
            references = self.vulnerability_references.search_vulnerability_references(vulnerability_id)
            print(references)
            attributes = self.vulnerability_extraction.extract_vulnerability_attributes(vulnerability_id)

            if attributes:
                print(attributes)
                annotation = self.vulnerability_annotation.annotate_vulnerability_attributes(vulnerability_id)
                print(annotation)
                locator = self.vulnerability_analysis.analyze_vulnerability_attributes(vulnerability_id)
                print(locator)
                return locator

        return None



from sator_core.ports.driven.gateways.oss import OSSGatewayPort
from sator_core.ports.driven.persistence.storage import StoragePersistencePort


class BaseBuilder:
    def __init__(self, oss_gateway: OSSGatewayPort, storage_port: StoragePersistencePort):
        self.storage_port = storage_port
        # TODO: temporary solution, should be moved somewhere else
        self.oss_gateway = oss_gateway



from sator_app.bootstrap.base import BaseBuilder

from sator_core.use_cases.annotation.attributes import (
    PatchAttributesAnnotation, ProductAttributesAnnotation, VulnerabilityAttributesAnnotation
)


from sator_core.ports.driven.classifiers.diff import DiffClassifierPort
from sator_core.ports.driven.classifiers.impact import ImpactClassifierPort
from sator_core.ports.driven.classifiers.product import ProductClassifierPort
from sator_core.ports.driven.classifiers.weakness import WeaknessClassifierPort
from sator_core.ports.driven.classifiers.patch_action import PatchActionClassifierPort


class AnnotationBuilder(BaseBuilder):
    def __init__(
            self, product_classifier: ProductClassifierPort, weakness_classifier: WeaknessClassifierPort,
            patch_action_classifier: PatchActionClassifierPort, impact_classifier: ImpactClassifierPort,
            diff_classifier: DiffClassifierPort, **kwargs
    ):
        super().__init__(**kwargs)

        self.diff_classifier = diff_classifier
        self.impact_classifier = impact_classifier
        self.product_classifier = product_classifier
        self.weakness_classifier = weakness_classifier
        self.patch_action_classifier = patch_action_classifier

    def create_vulnerability_attributes_annotation(self) -> VulnerabilityAttributesAnnotation:
        return VulnerabilityAttributesAnnotation(
            weakness_classifier=self.weakness_classifier,
            impact_classifier=self.impact_classifier,
            storage_port=self.storage_port
        )

    def create_product_attributes_annotation(self) -> ProductAttributesAnnotation:
        return ProductAttributesAnnotation(
            product_classifier=self.product_classifier,
            storage_port=self.storage_port
        )

    def create_patch_attributes_annotation(self) -> PatchAttributesAnnotation:
        return PatchAttributesAnnotation(
            patch_action_classifier=self.patch_action_classifier,
            weakness_classifier=self.weakness_classifier,
            diff_classifier=self.diff_classifier,
            storage_port=self.storage_port
        )
